@@ -5,33 +5,140 @@ Copyright (c) 2010, Jeremiah Dodds <jeremiah.dodds@gmail.com>
 
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the conditions in LICENSE.txt are met
+
+Provides a utility for retrieving the closest known "human readable" name for a
+color specified as a hex string. It does this by using an algorithm ported from
+ntc.js ( http://chir.ag/projects/ntc ). 
+
+There are 3 different sets of color definitions to choose from:
+  Resene ( http://people.csail.mit.edu/jaffer/Color/resenecolours.txt )
+  html4 ( http://www.w3.org/TR/css3-color/#html4 )
+  css3 ( http://www.w3.org/TR/css3-color/#svg-color )
+
+css3 is a superset of html4.
+
+The user can also supply their own colorfile to use. It should be comma
+separated hex,name pairs, and should include the leading #. Commas are allowed
+in names.
+
+Currently two output formats are supported -- json, and raw. 
+
+Usage:
+
+As a command-line utility:
+
+$ namethatcolor aabbcc
+{"hex_value": "#B0C4DE", "name": "lightsteelblue"}
+$ namethatcolor --color-set resene aabbcc
+{"hex_value": "#ADBED1", "name": "Casper"}
+
+#same as above
+$ namethatcolor -s resene aabbcc
+
+#specify your own color file
+$ namethatcolor -c path/to/color/file aabbcc
+
+As a library:
+
+>>> from namethatcolor import NameThatColor
+>>> Namer = NameThatColor()
+>>> Namer.name('aabbcc')
+Match('#B0C4DE', 'lightsteelblue', False, '#AABBCC')
+>>> resene_filename = NameThatColor.color_sets['resene']
+>>> resene_colors = NameThatColor.get_color_file(resene_filename)
+>>> Namer = NameThatColor(color_file=resene_colors)
+>>> Namer.name('aabbcc')
+Match('#ADBED1', 'Casper', False, '#AABBCC')
 """
+import json
 from collections import namedtuple
 
 ColorInfo = namedtuple('ColorInfo',
                        ' '.join(['hex_value', 'name', 'red', 'green', 'blue',
                                  'hue', 'saturation', 'lightness']))
-Match = namedtuple('Match', ' '.join(['hex_value', 'name', 'exact',
-                                      'original']))
 RGB = namedtuple('RGB', ' '.join(['red', 'green', 'blue']))
 HSL = namedtuple('HSL', ' '.join(['hue', 'saturation', 'lightness']))
 
+class Match(object):
+    """Provide a representation of the color match found for a hex string.
+    """
+    default_format = 'json'
+    default_output = ['hex_value', 'name']
+
+    formats = {
+        'json': lambda r: json.dumps(
+            dict([(k, r.__dict__[k]) for k in r.output])),
+        'raw' : lambda r: repr(r)
+    }
+
+    outputs = ['hex_value', 'name', 'exact', 'original']
+
+    def __init__(self, hex_value, name, exact, original, format_type=None,
+                 output=None):
+        if format_type is None:
+            format_type = self.default_format
+        if output is None:
+            output = self.default_output
+
+        self.output = output
+        self.format_type = format_type
+        self.hex_value = hex_value
+        self.name = name
+        self.exact = exact
+        self.original = original
+
+    def __repr__(self):
+        return "Match('%s', '%s', %s, '%s')" % (
+            self.hex_value, self.name, self.exact, self.original)
+
+    def __str__(self):
+        return self.formats[self.format_type](self)
 
 class NameThatColor(object):
     """Utility for finding the closest "human readable" name for a hex color
     """
-    def __init__(self, color_file):
-        import csv
+    default_color_set = 'css3'
+
+    color_sets = {
+        'resene': 'resene.csv',
+        'html4': 'html4.csv',
+        'css3': 'css3.csv'
+    }
+
+    def __init__(self, color_file=None, format=None, output=None):
+        if color_file is None:
+            color_file = self.get_color_file(
+                self.color_sets[self.default_color_set])
+
         self.color_info = []
 
-        reader = csv.reader(open(color_file))
+        if format is not None:
+            Match.default_format = format
+        if output is not None:
+            Match.default_output = output
 
-        for hex_val, name in reader:
-            red, green, blue = self.rgb(hex_val.strip())
-            hue, saturation, lightness = self.hsl(hex_val.strip())
-            self.color_info.append(ColorInfo(hex_val.strip(), name.strip(),
-                                             red, green, blue,
-                                             hue, saturation, lightness))
+        with open(color_file, 'r') as color_handle:
+            for line in [l for l in color_handle.readlines()
+                         if  l.startswith('#')]:
+                #there may be commas in names. this is a little sloppy.
+                parts = line.split(',')
+                hex_val = parts[0]
+                name = ','.join(parts[1:])
+
+                red, green, blue = self.rgb(hex_val.strip())
+                hue, saturation, lightness = self.hsl(hex_val.strip())
+                self.color_info.append(ColorInfo(hex_val.strip(), name.strip(),
+                                                 red, green, blue,
+                                                 hue, saturation, lightness))
+    @classmethod
+    def get_color_file(cls, color_file):
+        """Find the package resource for a color file.
+        """
+        from pkg_resources import Requirement, resource_filename
+        return resource_filename(
+            Requirement.parse('namethatcolor'),
+            'namethatcolor/data/%s' % color_file)
+    
 
     def name(self, color):
         """Return the closest human readable name given a color
@@ -54,7 +161,7 @@ class NameThatColor(object):
         ndf1 = 0
         ndf2 = 0
         ndf = 0
-        the_color = Match(None, None, None, None)
+        the_color = None
         df = -1
 
         for info in self.color_info:
@@ -76,10 +183,7 @@ class NameThatColor(object):
         if not the_color.name:
             return Match("#000000", "Invalid Color", False, color)
         else:
-            return Match(the_color.hex_value,
-                         the_color.name,
-                         False,
-                         color)
+            return Match(the_color.hex_value, the_color.name, False, color)
 
     def rgb(self, color):
         """Given a hex string representing a color, return an object with
@@ -130,36 +234,17 @@ def main():
     """Entry point for NameThatColor.py.
     Parse options, and find that color.
     """
-
-    import json
     import argparse
-
-    output_choices = {
-        'match_hex': lambda m: m.hex_value,
-        'match_name': lambda m: m.name,
-        'is_exact': lambda m: m.exact,
-        'original_hex': lambda m: m.original
-    }
-
-    format_choices = {
-        'json': lambda r: json.dumps(r),
-        'raw' : lambda r: r
-    }
-
-    color_set_choices = {
-        'resene': 'resene.csv',
-        'html4': 'html4.csv',
-        'css3': 'css3.csv'
-    }
     
     parser = argparse.ArgumentParser(
         description="Find the closest known color name for a hex value")
 
-    color_set_sources = parser.add_mutually_exclusive_group(required=True)
+    color_set_sources = parser.add_mutually_exclusive_group()
 
     color_set_sources.add_argument('-s', '--color-set', dest="color_set",
-                                  choices=color_set_choices.keys(),
-                                  help="the set of color names to match to.")
+                                   choices=NameThatColor.color_sets.keys(),
+                                   default=NameThatColor.default_color_set,
+                                   help="the set of color names to match to.")
 
     color_set_sources.add_argument('-c', '--colors', dest='colors_file',
                                    help="a csv file colors and their hex value")
@@ -167,14 +252,14 @@ def main():
     parser.add_argument('-o', '--output',
                         dest="output",
                         nargs='*',
-                        choices=output_choices.keys(),
-                        default=['match_hex', 'match_name'],
+                        choices=Match.outputs,
+                        default=Match.default_output,
                         help="what information about the color match to output")
 
     parser.add_argument('--format',
                         dest="format",
-                        choices=format_choices.keys(),
-                        default="json",
+                        choices=Match.formats.keys(),
+                        default=Match.default_format,
                         help="what format to return data in")
 
     parser.add_argument('target',
@@ -185,18 +270,12 @@ def main():
     colors_file = args.colors_file
 
     if not colors_file:
-        from pkg_resources import Requirement, resource_filename
-        colors_file = resource_filename(
-            Requirement.parse('namethatcolor'),
-            'namethatcolor/data/%s' % color_set_choices[args.color_set]
-        )
+        colors_file = NameThatColor.get_color_file(
+            NameThatColor.color_sets[args.color_set])
         
-    Namer = NameThatColor(colors_file)
-    match = Namer.name(args.target)
-    result = {}
-    for choice in args.output:
-        result[choice] = output_choices[choice](match)
-    print format_choices[args.format](result)
+    Namer = NameThatColor(colors_file, args.format, args.output)
+    print(Namer.name(args.target))
+
 
 if __name__ == '__main__':
     main()
